<!DOCTYPE html>
<html>
<head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8">
	<title>jQuery Test</title>
	<style>
	.hide {display:none;}
	#v {font-weight:bold;}
	</style>
	<script src="//ajax.googleapis.com/ajax/libs/jquery/2.0.3/jquery.min.js"></script>
	<script type="text/javascript">
		$(document).ready(function() {
			$('p.hide').show();
			$('#v').append($.fn.jquery);
		});
	</script>
</head>
<body>
	<h1>jQuery Test</h1>
	<p class="hide">If you see this, jQuery <span id="v"></span> is working.</p>
</body>
</html>
