{% load i18n %}

<h4>{% trans "View information" %}</h4>
<table>
	<thead>
		<tr>
			<th>{% trans "View function" %}</th>
			<th>{% trans "Arguments" %}</th>
			<th>{% trans "Keyword arguments" %}</th>
			<th>{% trans "URL name" %}</th>
		</tr>
	</thead>
	<tbody>
		<tr>
			<td class="djDebugOdd"><code>{{ view_func }}</code></td>
			<td class="djDebugEven"><code>{{ view_args|pprint }}</code></td>
			<td class="djDebugOdd"><code>{{ view_kwargs|pprint }}</code></td>
			<td class="djDebugEven"><code>{{ view_urlname }}</code></td>
		</tr>
	</tbody>
</table>

{% if cookies %}
	<h4>{% trans "Cookies" %}</h4>
	<table>
		<colgroup>
			<col style="width:20%"/>
			<col/>
		</colgroup>
		<thead>
			<tr>
				<th>{% trans "Variable" %}</th>
				<th>{% trans "Value" %}</th>
			</tr>
		</thead>
		<tbody>
			{% for key, value in cookies %}
				<tr class="{% cycle 'djDebugOdd' 'djDebugEven' %}">
					<td><code>{{ key|pprint }}</code></td>
					<td><code>{{ value|pprint }}</code></td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% else %}
	<h4>{% trans "No cookies" %}</h4>
{% endif %}

{% if session %}
	<h4>{% trans "Session data" %}</h4>
	<table>
		<colgroup>
			<col style="width:20%"/>
			<col/>
		</colgroup>
		<thead>
			<tr>
				<th>{% trans "Variable" %}</th>
				<th>{% trans "Value" %}</th>
			</tr>
		</thead>
		<tbody>
			{% for key, value in session %}
				<tr class="{% cycle 'djDebugOdd' 'djDebugEven' %}">
					<td><code>{{ key|pprint }}</code></td>
					<td><code>{{ value|pprint }}</code></td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% else %}
	<h4>{% trans "No session data" %}</h4>
{% endif %}

{% if get %}
	<h4>{% trans "GET data" %}</h4>
	<table>
		<colgroup>
			<col style="width:20%"/>
			<col/>
		</colgroup>
		<thead>
			<tr>
				<th>{% trans "Variable" %}</th>
				<th>{% trans "Value" %}</th>
			</tr>
		</thead>
		<tbody>
			{% for key, value in get %}
				<tr class="{% cycle 'djDebugOdd' 'djDebugEven' %}">
					<td><code>{{ key|pprint }}</code></td>
					<td><code>{{ value|pprint }}</code></td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% else %}
	<h4>{% trans "No GET data" %}</h4>
{% endif %}

{% if post %}
	<h4>{% trans "POST data" %}</h4>
	<table>
		<colgroup>
			<col style="width:20%"/>
			<col/>
		</colgr
			<tr>
				<th>{% trans "Variable" %}</th>
				<th>{% trans "Value" %}</th>
			</tr>
		</thead>
		<tbody>
			{% for key, value in post %}
				<tr class="{% cycle 'row1' 'row2' %}">
					<td><code>{{ key|pprint }}</code></td>
					<td><code>{{ value|pprint }}</code></td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% else %}
	<h4>{% trans "No POST data" %}</h4>
{% endif %}
