{% load i18n %}
<table>
	<thead>
		<tr>
			<th>{% trans "Signal" %}</th>
			<th>{% trans "Providing" %}</th>
			<th>{% trans "Receivers" %}</th>
		</tr>
	</thead>
	<tbody>
		{% for name, signal, receivers in signals %}
			<tr class="{% cycle 'djDebugOdd' 'djDebugEven' %}">
				<td>{{ name|escape }}</td>
				<td>{{ signal.providing_args|join:", " }}</td>
				<td>{{ receivers|join:", " }}</td>
			</tr>
		{% endfor %}
	</tbody>
</table>
