{% load i18n %}
{% if records %}
	<table>
		<thead>
			<tr>
				<th>{% trans "Level" %}</th>
				<th>{% trans "Time" %}</th>
				<th>{% trans "Channel" %}</th>
				<th>{% trans "Message" %}</th>
				<th>{% trans "Location" %}</th>
			</tr>
		</thead>
		<tbody>
			{% for record in records %}
				<tr class="{% cycle 'djDebugOdd' 'djDebugEven' %}">
					<td>{{ record.level }}</td>
					<td>{{ record.time|date:"h:i:s m/d/Y" }}</td>
					<td>{{ record.channel|default:"-" }}</td>
					<td>{{ record.message|linebreaksbr }}</td>
					<td>{{ record.file }}:{{ record.line }}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% else %}
	<p>{% trans "No messages logged" %}.</p>
{% endif %}

