{% load cache %}
<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <title>Index of Tests</title>
</head>
<body>
    <h1>Index of Tests</h1>
    {% cache 10 index_cache %}
    <ul>
        <li><a href="/jquery/">jQuery 2.0.3</a></li>
        <li><a href="/mootools/">MooTools 1.4.5</a></li>
        <li><a href="/prototype/">Prototype 1.7.1.0</a></li>
    </ul>
    <p><a href="/admin/">Django Admin</a></p>
    {% endcache %}
</body>
</html>
