{% load i18n %}
<!DOCTYPE html>
<html>
<head>
</head>
<body>
<h1>{{ status_line }}</h1>
<h2>{% trans "Location:" %} <a id="redirect_to" href="{{ redirect_to }}">{{ redirect_to }}</a></h2>
<p class="notice">
	{% trans "The Django Debug Toolbar has intercepted a redirect to the above URL for debug viewing purposes. You can click the above link to continue with the redirect as normal." %}
</p>
<script type="text/javascript">
    document.getElementById('redirect_to').focus();
</script>
</body>
</html>
