<!DOCTYPE html>
<html>
<head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8">
	<title>Prototype Test</title>
	<style>
	.hide {display:none;}
	</style>
	<script src="//ajax.googleapis.com/ajax/libs/prototype/1.7.1.0/prototype.js"></script>
	<script type="text/javascript">
		document.observe('dom:loaded', function() {
			$('showme').removeClassName('hide');
		});
	</script>
</head>
<body>
	<h1>Prototype Test</h1>
	<p class="hide" id="showme">If you see this, Prototype is working.</p>
</body>
</html>

