{% load i18n %}
<table>
	<thead>
		<tr>
			<th>{% trans "Setting" %}</th>
			<th>{% trans "Value" %}</th>
		</tr>
	</thead>
	<tbody>
		{% for name, value in settings.items %}
			<tr class="{% cycle 'djDebugOdd' 'djDebugEven' %}">
				<td>{{ name }}</td>
				<td><code>{{ value|pprint }}</code></td>
			</tr>
		{% endfor %}
	</tbody>
</table>
